@@ -6,7 +6,7 @@ Mojolicious::Plugin::AssetPack - Compress and convert css, less, sass and javasc
 
 =head1 VERSION
 
-0.0402
+0.05
 
 =head1 SYNOPSIS
 
@@ -14,7 +14,7 @@ In your application:
 
   use Mojolicious::Lite;
 
-  plugin AssetPack => { rebuild => 1 };
+  plugin 'AssetPack';
 
   # add a preprocessor
   app->asset->preprocessors->add(js => sub {
@@ -33,6 +33,12 @@ In your template:
   %= asset 'app.js'
   %= asset 'app.css'
 
+Or if you need to add the tags manually:
+
+  % for my $asset (asset->get('app.js')) {
+    %= javascript $asset
+  % }
+
 See also L</register>.
 
 =head1 DESCRIPTION
@@ -81,13 +87,12 @@ details.
 
 use Mojo::Base 'Mojolicious::Plugin';
 use Mojo::ByteStream 'b';
-use Mojo::Util qw( md5_sum slurp spurt );
+use Mojo::Util;
 use Mojolicious::Plugin::AssetPack::Preprocessors;
-use Fcntl qw( O_CREAT O_EXCL O_WRONLY );
 use File::Basename qw( basename );
 use File::Spec::Functions qw( catfile );
 
-our $VERSION = '0.0402';
+our $VERSION = '0.05';
 
 =head1 ATTRIBUTES
 
@@ -109,11 +114,13 @@ has preprocessors => sub { Mojolicious::Plugin::AssetPack::Preprocessors->new };
 
 =head2 rebuild
 
-Set this to true if the assets should created, even though they exist.
+Deprecated.
 
 =cut
 
-has rebuild => 0;
+sub rebuild {
+  warn "rebuild() has no effect any more. Will soon be removed."
+}
 
 =head1 METHODS
 
@@ -129,8 +136,21 @@ helper is called on the app.
 sub add {
   my($self, $moniker, @files) = @_;
 
-  $self->{assets}{$moniker} = [@files];
-  $self->process($moniker) if $self->minify;
+  $self->{assets}{$moniker} = \@files;
+
+  if($self->minify) {
+    $self->process($moniker => @files);
+  }
+  else {
+    for my $file (@files) {
+      next unless $file =~ /\.(less|s[ac]ss)$/;
+      my $moniker = basename $file;
+      $moniker =~ s/\.\w+$/.css/;
+      $self->process($moniker => $file);
+      $file = delete $self->{assets}{$moniker};
+    }
+  }
+
   $self;
 }
 
@@ -151,20 +171,38 @@ sub expand {
   my($self, $c, $moniker) = @_;
   my $files = $self->{assets}{$moniker};
 
-  if(!$files) {
-    return b "<!-- Could not expand $moniker -->";
+  if(!ref $files) {
+    return b "<!-- Cannot expand $moniker -->";
   }
   elsif($moniker =~ /\.js/) {
     return b join "\n", map { $c->javascript($_) } @$files;
   }
   else {
-    return b join "\n", map { $c->stylesheet($self->_compile_css($_)) } @$files;
+    return b join "\n", map { $c->stylesheet($_) } @$files;
   }
 }
 
+=head2 get
+
+  @files = $self->get($moniker);
+
+Returns a list of files which the moniker point to. The list will only
+contain one file if the C<$moniker> is minified.
+
+=cut
+
+sub get {
+  my($self, $moniker) = @_;
+  my $files = $self->{assets}{$moniker};
+
+  return unless $files;
+  return @$files if ref $files;
+  return $files;
+}
+
 =head2 process
 
-  $self->process($moniker);
+  $self->process($moniker => @files);
 
 This method use L<Mojolicious::Plugin::AssetPack::Preprocessors/process> to
 convert and/or minify the sources pointed at by C<$moniker>.
@@ -174,45 +212,54 @@ The result file will be stored in L</Packed directory>.
 =cut
 
 sub process {
-  my($self, $moniker) = @_;
-  my $assets = $self->{assets}{$moniker};
-  my $mode = $self->rebuild ? O_CREAT | O_WRONLY : O_CREAT | O_EXCL | O_WRONLY;
-  my $out_file = catfile $self->{out_dir}, $moniker;
-  my $doc = '';
-  my($fh, @missing);
+  my($self, $moniker, @files) = @_;
+  my($md5_sum, $out, $out_file, @missing);
+  my $content = {};
 
-  for my $asset (@$assets) {
-    die "Missing valid extension from asset $asset" unless $asset =~ /\.(\w{1,4})$/;
-    push @missing, $asset unless @{ $self->preprocessors->subscribers($1) || [] };
+  # @files will contain full path after this map {}
+  $md5_sum = Mojo::Util::md5_sum(
+              join '', map {
+                my $file = $self->{static}->file($_);
+                $_ = $file->path if $file;
+                $content->{$_} = Mojo::Util::slurp($_);
+              } @files
+             );
+
+  $out_file = $moniker;
+  $out_file =~ s/\.(\w{1,4})$/-$md5_sum.$1/;
+
+  if(-s catfile $self->{out_dir}, $out_file) {
+    $self->{log}->debug("Using existing asset for $moniker");
+    $self->{assets}{$moniker} = "/packed/$out_file";
+    return $self;
+  }
+
+  for my $file (@files) {
+    next if $file =~ /\.(\w{1,4})$/ and $self->preprocessors->has_subscribers($1);
+    push @missing, $file; # will also contain files without extensions
   }
 
   if(@missing) {
-    $self->{log}->debug("Missing preprocessors for @missing");
-    $self->_find_processed($moniker);
-    return;
-  }
-  if(!($fh = IO::File->new($out_file, $mode))) {
-    $self->{log}->debug("Could not write $out_file: $!");
-    $self->_find_processed($moniker);
-    return;
+    local $" = "\n- ";
+    die "AssetPack missing preprocessors:\n- @missing\n";
   }
 
-  for my $asset (@$assets) {
-    my $extension = $asset =~ /\.(\w{1,4})$/ ? $1 : '';
-    my $file = $self->{static}->file($asset);
-    my $text;
-
-    $file = $file ? $file->path : $asset;
-    $text = slurp $file;
-    $self->preprocessors->process($extension, $self, \$text, $file);
-    $doc .= $text;
-  }
-
-  $fh->truncate(0);
-  $fh->syswrite($doc);
-  $fh->close or die "close $out_file: $!";
   $self->_remove_processed($moniker) if $self->{cleanup};
-  $self->_rename_processed($moniker, md5_sum $doc);
+
+  Mojo::Util::spurt(
+    join('',
+      map {
+        /\.(\w{1,4})$/; # checked in @missing loop
+        $self->preprocessors->process($1, $self, \$content->{$_}, $_);
+        $content->{$_};
+      } @files
+    ),
+    catfile($self->{out_dir}, $out_file)
+  );
+
+  $self->{log}->debug("Built asset for $moniker ($out_file)");
+  $self->{assets}{$moniker} = "/packed/$out_file";
+  return $self;
 }
 
 =head2 register
@@ -221,7 +268,6 @@ sub process {
     cleanup => $bool, # default is true
     minify => $bool, # compress assets
     no_autodetect => $bool, # disable preprocessor autodetection
-    rebuild => $bool, # overwrite if assets exists
   };
 
 Will register the C<compress> helper. All arguments are optional.
@@ -231,9 +277,6 @@ have other web sites that need to access an old version of the minified files.
 
 "minify" will default to true if L<Mojolicious/mode> is "production".
 
-"rebuild" can be set to true to always rebuild the compressed files when the
-application is started. The default is to use the cached files.
-
 =cut
 
 sub register {
@@ -242,7 +285,6 @@ sub register {
   my $helper = $config->{helper} || 'asset';
 
   $self->minify($minify);
-  $self->rebuild($config->{rebuild} || 0);
   $self->preprocessors->detect unless $config->{no_autodetect};
 
   $self->{assets} = {};
@@ -257,52 +299,11 @@ sub register {
     return $self if @_ == 1;
     return shift, $self->add(@_) if @_ > 2;
     return $self->expand(@_) unless $minify;
-    return $_[0]->javascript("/packed/$self->{assets}{$_[1]}") if $_[1] =~ /\.js$/;
-    return $_[0]->stylesheet("/packed/$self->{assets}{$_[1]}");
+    return $_[0]->javascript($self->{assets}{$_[1]}) if $_[1] =~ /\.js$/;
+    return $_[0]->stylesheet($self->{assets}{$_[1]});
   });
 }
 
-sub _compile_css {
-  my($self, $file) = @_;
-  my $original = $file;
-
-  if($file =~ s/\.(scss|less)$/.css/) {
-    eval {
-      my $extension = $1;
-      my $in = $self->{static}->file($original)->path;
-      (my $out = $in) =~ s/\.\w+$/.css/;
-      my $text = slurp $in;
-      $self->preprocessors->process($extension, $self, \$text, $in);
-      spurt $text, $out;
-      1;
-    } or do {
-      $self->{log}->warn("Could not convert $original: $@");
-    };
-  }
-
-  $file;
-}
-
-sub _find_processed {
-  my($self, $moniker) = @_;
-
-  $self->{processed} ||= do {
-    opendir(my $DH, $self->{out_dir});
-    +{
-      map {
-        ($_->[0] => $_->[1]);
-      } sort {
-        $a->[11] <=> $b->[11]; # mtime
-      } map {
-        my @m = m!^(.+)-\w{32}\.(.+)!;
-        [ join('.', @m) => $_ => stat catfile $self->{out_dir}, $_ ];
-      } readdir $DH
-    };
-  };
-
-  $self->{assets}{$moniker} = $self->{processed}{$moniker} || "not-processed-$moniker";
-}
-
 sub _remove_processed {
   my($self, $moniker) = @_;
   my($name, $ext) = $moniker =~ m!^(.+)\.(\w+)$!;
@@ -315,17 +316,6 @@ sub _remove_processed {
   }
 }
 
-sub _rename_processed {
-  my($self, $moniker, $checksum) = @_;
-  my($name, $ext) = $moniker =~ m!^(.+)\.(\w+)$!;
-  my $source = catfile $self->{out_dir}, $moniker;
-  my $destination = catfile $self->{out_dir}, "$name-$checksum.$ext";
-
-  $self->{assets}{$moniker} = "$name-$checksum.$ext";
-  unlink $destination if -e $destination;
-  rename $source, $destination or die "Could not rename $source to $destination: $!";
-}
-
 =head1 AUTHOR
 
 Jan Henning Thorsen - C<jhthorsen@cpan.org>
